import os

import imagehash
import pytest
from PIL import Image

from jumpstarter_imagehash import ImageHash


def _image_path(filename):
    current_file_directory = os.path.dirname(__file__)
    return os.path.join(current_file_directory, filename)


def test_imagehash_assert_snapshot():
    snapshot_a = SnapshotMock("test_image_a.jpeg")
    ImageHash(snapshot_a).assert_snapshot(_image_path("test_image_a.jpeg"))  # ty: ignore[invalid-argument-type]


def test_imagehash_fail_assert_snapshot(tmp_path):
    snapshot_a = SnapshotMock("test_image_a.jpeg")
    # this should raise an AssertionError
    with pytest.raises(AssertionError):
        ImageHash(snapshot_a, outdir=tmp_path).assert_snapshot(  # ty: ignore[invalid-argument-type]
            _image_path("test_image_b.jpeg")
        )
    assert (tmp_path / "FAILED_test_image_b.jpeg").exists()


def test_imagehash_passthrough_snapshot():
    snapshot_a = SnapshotMock("test_image_a.jpeg")
    assert ImageHash(snapshot_a).snapshot() == snapshot_a.img  # ty: ignore[invalid-argument-type]


def test_imagehash_hash_snapshot():
    snapshot_a = SnapshotMock("test_image_a.jpeg")
    assert isinstance(ImageHash(snapshot_a).hash_snapshot(), imagehash.ImageHash)  # ty: ignore[invalid-argument-type]


class SnapshotMock:
    def __init__(self, filename):
        self.name = "SnapshotMock"
        self.img = Image.open(_image_path(filename))

    def snapshot(self):
        return self.img



import logging
import os
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any

import imagehash
from PIL import Image

from jumpstarter.client import DriverClient

log = logging.getLogger("imagehash")


@dataclass
class ImageHash:
    """ImageHash class

    ImageHash class for image hashing and comparison

    param client: video client object, must provide a snapshot method
    type client: object
    param outdir: where to store actual image files, default is Path.cwd()
    type outdir: Path
    param hash_func: hash function to use from imagehash library, default is average_hash
    type hash_func: function
    param hash_size: hash size to use from imagehash library, default is 8
    type hash_size: int
    """

    client: DriverClient
    outdir: Path = field(default_factory=Path.cwd)
    hash_func: Any = imagehash.average_hash
    hash_size: int = 8

    def snapshot(self):
        """
        Get a snapshot image from the video input

        :return: a snapshot image
        :rtype: PIL.Image
        """
        return self.client.snapshot()

    def hash_snapshot(self):
        """
        Get a hash of the snapshot image through the imagehash library

        :return: hash of the snapshot image
        :rtype: ImageHash
        """
        return self.hash_func(self.snapshot())

    def assert_snapshot(self, reference_img_file, tolerance=1):
        """
        Assert the snapshot image is the same as the reference image

        :param str reference_img_file: reference image file name
        :param int tolerance: hash difference tolerance

        :raises AssertionError: if the snapshot image is different from the reference image
        """
        diff, snapshot_img = self._snapshot_diff(reference_img_file)
        if diff > tolerance:
            reference_img_file = "FAILED_" + os.path.basename(reference_img_file)

            save_filename = self.outdir / reference_img_file
            log.error(f"Image hashes are different, saving the actual image as {save_filename}")
            snapshot_img.save(save_filename)
            raise AssertionError(
                f"{self.client.name}.assert_snapshot {reference_img_file}: diff {diff} > tolerance {tolerance}"
            )

    def _snapshot_diff(self, reference_img_file, hash_func=imagehash.average_hash, hash_size=8):
        snapshot_img = self.snapshot()
        ref_hash = self.hash_func(Image.open(reference_img_file), hash_size=self.hash_size)
        snapshot_hash = self.hash_func(snapshot_img, hash_size=self.hash_size)
        diff = ref_hash - snapshot_hash
        log.info(
            f"{self.client.name} comparing snapshot {reference_img_file}:"
            f" snapshot {snapshot_hash}, ref {ref_hash}, diff: {diff}"
        )
        return diff, snapshot_img
